#ifndef __SECPOLICY__ /*{*/
#define __SECPOLICY__

/*
 * @(#)secpolicy.h	2.6.2.3 14:35:33 12/27/89 SecureWare
 *
 * @(#)secpolicy.h 21.1.1.2 94/03/02 SMI; SunOS CMW
 * Copyright (c) 1988-1989 SecureWare, Inc.  All rights reserved.
 *
 * This is unpublished proprietary source code of SecureWare, Inc. and
 * should be treated as confidential.
 */

#ifdef SunOS_CMW

/***************************************************

  #ifdefs for other vendors' systems have been
  removed from this file - too hard to read.

 ***************************************************/

#if defined(TNET_DBASE)  /*{*/
/*
 * The Trusted Network Token Database uses tags that are two longwords longer
 * than other tags.  The extra longs store the remote host ID and attribute
 * ID.
 */
#define SEC_TAG_SIZE	3		/* Tag Size in Longwords */
#else	/*}{*/
#define SEC_TAG_SIZE	1		/* Tag Size in Longwords */
#endif	/*}*/

/* mh_flag for spd_send_message to indicate no wait */
#define TNET_NO_RESP	0x400
#define TNET_NO_LABELD	0x800

/*	Configurable Maximum Number of Security Tags Per Object		*/

#define SEC_TAG_COUNT		8		/* maximum object tags */


/* System-wide definition for wildcard tag */

#define SEC_WILDCARD_TAG_VALUE	0

#ifndef SunOS_CMW
/* need real definition from SecureWare */
struct mand_config {
	tag_t first_subj_tag;
} mand_config;
#endif /* !SunOS_CMW */

/* Internal Representation Typedefs */

/* User Space to Kernel Space */

typedef struct {
	unchar	code;			/* function/return code value */
	ushort	ir_length;		/* length of internal representation */
	char	*ir;			/* internal representation pointer */
} attr_t;

/* Internal Representation in Messages */

typedef struct {
	ulong	ir_length;		/* length of internal representation */

/*	char	rep[];			 * comment-internal representation */
} ir_t;

#define IR_T_SIZE	(sizeof(ir_t))

/*	Configurable Maximum of IR Size	    */

#define SEC_MAX_IR_SIZE		8192		/* sanity check */

/* Attribute Type Typedef */

typedef short attrtype_t;

/* Security Policy Module and Security Policy Daemon Message Types */

#define SPD_MAP_TAG		1	/* Map Tag to IR */
#define SPD_SET_TAG		2	/* Set Tag on Object */
#define SPD_MAKE_DECISION	3	/* Make an Access Decision */
#define	SPD_DECISION		4	/* Decision from Daemon */
#define SPD_GET_ATTRIBUTE	5	/* Read IR for Tag */
#define SPD_INTERNAL_REP	6	/* Response with IR for Tag */
#define SPD_CHANGE_DISCR	7	/* Change Discretionary */
#define SPD_NEW_DISCR		8	/* Change Discretionary Response */
#define SPD_SHUTDOWN		9	/* Terminate the Daemon */
#define SPD_SHUTDOWN_RESP	10	/* ACK the Shutdown Request */
#define SPD_PREPARE_SHUTDOWN	11	/* Shutdown Notification */
#define SPD_PREPARE_RESP	12	/* Shutdown Notification Response */
#define SPD_ERROR		13	/* Error Message type */
#define SPD_OK			14

/* Security Policy Module/Daemon Message Formats */

/*********** Common Security Policy Message Header Format ************/

struct spd_message_header {
	long	sync_id;	/* process sync id for driver */
	short	msg_type;	/* request-response type */
	short	mh_flags;		/* state field */
	short	error_code;	/* error return for audit */
};

typedef struct spd_message_header mhdr_t;

#define SPD_MHDR_SIZE	(sizeof(struct spd_message_header))


/* Security Message Driver Ioctl Functions */

/*
 *	The BSD and AUX based driver ioctls require a special encoding.
 *	This encoding technique is not used on audit for the System V
 *	version of the driver. The definition of this macro will allow
 *	in the System V case the use of the same definition as BSD and AUX.
 */

#include "sys/ioccom.h"
 
#define SPIOC_INIT	_IO(A,1)	/* initialize the policy cache */
#define SPIOC_INV_CACHE	_IO(A,2)	/* invalidate the decision cache */
#define SPIOC_INV_TAG	_IO(A,3)	/* invalidate cache entry */
#define SPIOC_IMMUNE	_IO(A,4)	/* prepare for shutdown */
#define SPIOC_SHUTDOWN	_IO(A,5)	/* terminate */
#define SPIOC_SET_CACHE_SIZE	_IO(A,6)	/* alter the cache size */
#define SPIOC_GET_STATS	_IO(A,7)	/* retrieve policy statistics */
#define SPIOC_GETCONF	_IO(A,8)	/* retrieve policy configuration */
#define SPIOC_EXCL	_IO(A,9)	/* make daemon device exclusive */

/* Privileged Re-init Ioctl Function */

#define SPIOC_REINIT	_IO(A,20)	/* reinitialize spd structures */

/* Special tnet daemon ioctls */
#define TNIOC_LOAD_CONF _IO(A,30)     /* Load configuration parameters. */
#define TNIOC_SHUTDOWN_NET _IO(A,31)  /* Load configuration parameters. */
#define TNIOC_FLUSH_RH 	_IO(A,32)     /* Flush a remote host cache */
#define TNIOC_FLUSH_LOC _IO(A,33)     /* Flush local cache. */
#define TNIOC_LOAD_RH	_IO(A,34)     /* Load rh db record into kernel */
#define TNIOC_LOAD_SR	_IO(A,35)     /* Load sr db record into kernel */
#define TNIOC_CACHE_UPDATE _IO(A,36)  /* Load token map into kernel cache */
#define TNIOC_RESTART 	_IO(A,37)     /* Restart a kernel operation. */
#define TNIOC_GET_STATS _IO(A,38)     /* Get kernel statistics. */
#define TNIOC_POLICY_CHK _IO(A,39)    /* Make a policy decision. */

#ifdef SunOS_CMW
#define TNIOC_RH_BOOTING _IO(A,40)     /* Mark the specified host as booting */
#define TNIOC_RH_DONE_BOOTING _IO(A,41)
#endif


/* Ioctl Structure Definitions */

struct sp_init {
	ulong	cache_size;		/* number of cache entries */
	ulong	cache_width;		/* decision size */
	ushort	magic;			/* policy magic number */
	ushort	spminor;		/* policy minor device */
	ushort	policy;			/* policy number */
	ushort	first_subj_tag;		/* first subject tag */
	ushort	subj_tag_count;		/* subject tag count */
	ushort	first_obj_tag;		/* first object tag */
	ushort	obj_tag_count;		/* object tag count */

/*	char	cache_entry[];		 * comment-decision */
};

#define SP_INITSIZE	(sizeof(struct sp_init))

/* Ioctl Structure for Dynamically Altering the Cache Size */

struct sp_set_cache_size {
	ulong	cache_size;		/* new cache size */
	ulong	cache_width;		/* new cache width */
};

#ifndef SunOS_CMW
/* Ioctl Structure for Statistics Retrieval */

/* Module Specific Statistics */

struct sp_mod_stats {
	ulong	decisions;		/* decisions made */
	ulong	queries;		/* daemon queries made */
	ulong	responses;		/* responses from daemon */
	ulong	collisions;		/* cache entry collisions */
	ulong	delay;			/* aggregate delay */
	ulong	max_delay;		/* maximum decision delay */
	ulong	cache_size;		/* number of cache entries */
	ulong	cache_width;		/* decision size */
};

#define SP_MOD_STAT_SIZE	(sizeof(struct sp_mod_stats))

extern struct sp_mod_stats *sp_mod_stats;
#endif /* !SunOS_CMW */

#define SPD_CONTROL_MINOR       0xff            /* minor device number */

/*
 * Macro definitions to make spd_allocate() backwards compatible.
 */
#define spd_allocate(request_size) 	  spd_alloc_int((request_size), (int)1)
#define spd_allocate_nowait(request_size) spd_alloc_int((request_size), (int)0)


#endif /* SunOS_CMW */
#endif /* __SECPOLICY__ }*/
